from typing import List

from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy.orm import Session

from workspaces.system.schema import NotificationTemplate
from workspaces.system.services_interfaces import NotificationInfoIF

Base = declarative_base()


class NotificationInfo(NotificationInfoIF):
    def __init__(self, session: Session):
        self.session = session

    def all_templates(self) -> List[NotificationTemplate]:
        return self.session.query(NotificationTemplate).all()

    def lookup_template(self, template_name: str) -> NotificationTemplate:
        return (
            self.session.query(NotificationTemplate)
            .filter_by(name=template_name)
            .first()
        )

    def create_template(
        self, name: str, description: str, template: str
    ) -> NotificationTemplate:
        """
        Create new notification template and save it in the database

        :param name: Name of new template
        :param description: brief description of template
        :param template: the full text of the template
        :return: Created NotificationTemplate
        """
        template = NotificationTemplate(
            name=name, description=description, content=template
        )
        self.save_entity(template)
        return template

    def delete_template(self, template_name: str):
        template = (
            self.session.query(NotificationTemplate)
            .filter_by(name=template_name)
            .first()
        )
        self.session.delete(template)

    def save_entity(self, entity: Base):
        """
        Save entity to database

        :param entity:
        """
        self.session.add(entity)
        self.session.flush()
